<?php

namespace Biopen\SaasBundle\Controller;

use Sonata\AdminBundle\Controller\CRUDController as Controller;
use Symfony\Component\Process\Process;
use Sonata\AdminBundle\Datagrid\ProxyQueryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

class ProjectAdminController extends Controller
{
   /**
     * Delete action.
     *
     * @param int|string|null $id
     *
     * @return Response|RedirectResponse
     *
     * @throws NotFoundHttpException If the object does not exist
     * @throws AccessDeniedException If access is not granted
     */
    public function deleteAction($id)
    {
        $request = $this->getRequest();
        $id = $request->get($this->admin->getIdParameter());
        $object = $this->admin->getObject($id);

        if (!$object) throw $this->createNotFoundException(sprintf('unable to find the object with id : %s', $id));
        $this->admin->checkAccess('delete', $object);

        $preResponse = $this->preDelete($request, $object);
        if ($preResponse !== null) return $preResponse;

        if ($this->getRestMethod() == 'DELETE') {
            // check the csrf token
            $this->validateCsrfToken('sonata.delete');
            $objectName = $this->admin->toString($object);

            try {
                $this->admin->delete($object);
                $this->dropDatabase($object);
                $this->addFlash('sonata_flash_success',$this->trans('flash_delete_success',array('%name%' => $this->escapeHtml($objectName)),'SonataAdminBundle'));
            } catch (ModelManagerException $e) {
                $this->handleModelManagerException($e);
                $this->addFlash('sonata_flash_error',  $this->trans('flash_delete_error',array('%name%' => $this->escapeHtml($objectName)),'SonataAdminBundle'));
            }

            return $this->redirectTo($object);
        }

        return $this->render($this->admin->getTemplate('delete'), array(
            'object' => $object,
            'action' => 'delete',
            'csrf_token' => $this->getCsrfToken('sonata.delete'),
        ), null);
    }

    private function dropDatabase($project)
    {
      // Drop the database of this project
      $commandline = 'mongo ' . $project->getDbName() .' --eval "db.dropDatabase()"';
      $process = new Process($commandline);
      return $process->start();
    }

    /**
     * Execute a batch delete.
     *
     * @param ProxyQueryInterface $query
     *
     * @return RedirectResponse
     *
     * @throws AccessDeniedException If access is not granted
     */
    public function batchActionDelete(ProxyQueryInterface $query)
    {
        $this->admin->checkAccess('batchDelete');

        try {
            $this->batchDelete($this->admin->getClass(), $query);
            $this->addFlash('sonata_flash_success', 'flash_batch_delete_success');
        } catch (Exception $e) {
            $this->addFlash('sonata_flash_error', 'flash_batch_delete_error');
        }

        return new RedirectResponse($this->admin->generateUrl(
            'list',
            array('filter' => $this->admin->getFilterParameters())
        ));
    }

    private function batchDelete($class, ProxyQueryInterface $queryProxy)
    {
        $queryBuilder = $queryProxy->getQuery();
        $documentManager = $this->get('doctrine_mongodb')->getManager();

        $i = 0;
        foreach ($queryBuilder->execute() as $object) {
            $documentManager->remove($object);
            $this->dropDatabase($object);

            if ((++$i % 20) == 0) {
                $documentManager->flush();
                $documentManager->clear();
            }
        }

        $documentManager->flush();
        $documentManager->clear();
    }
}